@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from openstack.compute.v2 import aggregate as _aggregate
 from openstack.compute.v2 import availability_zone
 from openstack.compute.v2 import extension
 from openstack.compute.v2 import flavor as _flavor
@@ -18,6 +19,7 @@ from openstack.compute.v2 import image as _image
 from openstack.compute.v2 import keypair as _keypair
 from openstack.compute.v2 import limits
 from openstack.compute.v2 import server as _server
+from openstack.compute.v2 import server_diagnostics as _server_diagnostics
 from openstack.compute.v2 import server_group as _server_group
 from openstack.compute.v2 import server_interface as _server_interface
 from openstack.compute.v2 import server_ip
@@ -29,11 +31,203 @@ from openstack import resource
 
 class Proxy(proxy.Proxy):
 
-    Extension = extension.Extension
-    Flavor = _flavor.Flavor
-    FlavorDetail = _flavor.FlavorDetail
-    Server = _server.Server
-    ServerDetail = _server.ServerDetail
+    def find_extension(self, name_or_id, ignore_missing=True):
+        """Find a single extension
+
+        :param name_or_id: The name or ID of an extension.
+        :param bool ignore_missing: When set to ``False``
+                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    raised when the resource does not exist.
+                    When set to ``True``, None will be returned when
+                    attempting to find a nonexistent resource.
+        :returns: One :class:`~openstack.compute.v2.extension.Extension` or
+                  None
+        """
+        return self._find(extension.Extension, name_or_id,
+                          ignore_missing=ignore_missing)
+
+    def extensions(self):
+        """Retrieve a generator of extensions
+
+        :returns: A generator of extension instances.
+        :rtype: :class:`~openstack.compute.v2.extension.Extension`
+        """
+        return self._list(extension.Extension)
+
+    def find_flavor(self, name_or_id, ignore_missing=True):
+        """Find a single flavor
+
+        :param name_or_id: The name or ID of a flavor.
+        :param bool ignore_missing: When set to ``False``
+                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    raised when the resource does not exist.
+                    When set to ``True``, None will be returned when
+                    attempting to find a nonexistent resource.
+        :returns: One :class:`~openstack.compute.v2.flavor.Flavor` or None
+        """
+        return self._find(_flavor.Flavor, name_or_id,
+                          ignore_missing=ignore_missing)
+
+    def create_flavor(self, **attrs):
+        """Create a new flavor from attributes
+
+        :param dict attrs: Keyword arguments which will be used to create
+                           a :class:`~openstack.compute.v2.flavor.Flavor`,
+                           comprised of the properties on the Flavor class.
+
+        :returns: The results of flavor creation
+        :rtype: :class:`~openstack.compute.v2.flavor.Flavor`
+        """
+        return self._create(_flavor.Flavor, **attrs)
+
+    def delete_flavor(self, flavor, ignore_missing=True):
+        """Delete a flavor
+
+        :param flavor: The value can be either the ID of a flavor or a
+                       :class:`~openstack.compute.v2.flavor.Flavor` instance.
+        :param bool ignore_missing: When set to ``False``
+                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    raised when the flavor does not exist.
+                    When set to ``True``, no exception will be set when
+                    attempting to delete a nonexistent flavor.
+
+        :returns: ``None``
+        """
+        self._delete(_flavor.Flavor, flavor, ignore_missing=ignore_missing)
+
+    def get_flavor(self, flavor):
+        """Get a single flavor
+
+        :param flavor: The value can be the ID of a flavor or a
+                       :class:`~openstack.compute.v2.flavor.Flavor` instance.
+
+        :returns: One :class:`~openstack.compute.v2.flavor.Flavor`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        return self._get(_flavor.Flavor, flavor)
+
+    def flavors(self, details=True, **query):
+        """Return a generator of flavors
+
+        :param bool details: When ``True``, returns
+            :class:`~openstack.compute.v2.flavor.FlavorDetail` objects,
+            otherwise :class:`~openstack.compute.v2.flavor.Flavor`.
+            *Default: ``True``*
+        :param kwargs query: Optional query parameters to be sent to limit
+                                 the flavors being returned.
+
+        :returns: A generator of flavor objects
+        """
+        flv = _flavor.FlavorDetail if details else _flavor.Flavor
+        return self._list(flv, **query)
+
+    def aggregates(self):
+        """Return a generator of aggregate
+
+        :returns: A generator of aggregate
+        :rtype: class: `~openstack.compute.v2.aggregate.Aggregate`
+        """
+        aggregate = _aggregate.Aggregate
+
+        return self._list(aggregate)
+
+    def get_aggregate(self, aggregate):
+        """Get a single host aggregate
+
+        :param image: The value can be the ID of an aggregate or a
+                      :class:`~openstack.compute.v2.aggregate.Aggregate`
+                      instance.
+
+        :returns: One :class:`~openstack.compute.v2.aggregate.Aggregate`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        return self._get(_aggregate.Aggregate, aggregate)
+
+    def create_aggregate(self, **attrs):
+        """Create a new host aggregate from attributes
+
+        :param dict attrs: Keyword arguments which will be used to create a
+                           :class:`~openstack.compute.v2.aggregate.Aggregate`,
+                           comprised of the properties on the Aggregate class.
+
+        :returns: The results of aggregate creation
+        :rtype: :class:`~openstack.compute.v2.aggregate.Aggregate`
+        """
+        return self._create(_aggregate.Aggregate, **attrs)
+
+    def update_aggregate(self, aggregate, **attrs):
+        """Update a host aggregate
+
+        :param server: Either the ID of a host aggregate or a
+                       :class:`~openstack.compute.v2.aggregate.Aggregate`
+                       instance.
+        :attrs kwargs: The attributes to update on the aggregate represented
+                       by ``aggregate``.
+
+        :returns: The updated aggregate
+        :rtype: :class:`~openstack.compute.v2.aggregate.Aggregate`
+        """
+        return self._update(_aggregate.Aggregate, aggregate, **attrs)
+
+    def delete_aggregate(self, aggregate, ignore_missing=True):
+        """Delete a host aggregate
+
+        :param keypair: The value can be either the ID of an aggregate or a
+                        :class:`~openstack.compute.v2.aggregate.Aggregate`
+                        instance.
+        :param bool ignore_missing: When set to ``False``
+                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    raised when the aggregate does not exist.
+                    When set to ``True``, no exception will be set when
+                    attempting to delete a nonexistent aggregate.
+
+        :returns: ``None``
+        """
+        self._delete(_aggregate.Aggregate, aggregate,
+                     ignore_missing=ignore_missing)
+
+    def add_host_to_aggregate(self, aggregate, host):
+        """Adds a host to an aggregate
+
+        :param aggregate: Either the ID of a aggregate or a
+                    :class:`~openstack.compute.v2.aggregate.Aggregate`
+                    instance.
+        :param str host: The host to add to the aggregate
+
+        :returns: One :class:`~openstack.compute.v2.aggregate.Aggregate`
+        """
+        aggregate = self._get_resource(_aggregate.Aggregate, aggregate)
+        return aggregate.add_host(self, host)
+
+    def remove_host_from_aggregate(self, aggregate, host):
+        """Removes a host from an aggregate
+
+        :param aggregate: Either the ID of a aggregate or a
+                    :class:`~openstack.compute.v2.aggregate.Aggregate`
+                    instance.
+        :param str host: The host to remove from the aggregate
+
+        :returns: One :class:`~openstack.compute.v2.aggregate.Aggregate`
+        """
+        aggregate = self._get_resource(_aggregate.Aggregate, aggregate)
+        return aggregate.remove_host(self, host)
+
+    def set_aggregate_metadata(self, aggregate, metadata):
+        """Creates or replaces metadata for an aggregate
+
+        :param aggregate: Either the ID of a aggregate or a
+                    :class:`~openstack.compute.v2.aggregate.Aggregate`
+                    instance.
+        :param dict metadata: Metadata key and value pairs. The maximum
+                    size for each metadata key and value pair
+                    is 255 bytes.
+
+        :returns: One :class:`~openstack.compute.v2.aggregate.Aggregate`
+        """
+        aggregate = self._get_resource(_aggregate.Aggregate, aggregate)
+        return aggregate.set_metadata(self, metadata)
 
     def delete_image(self, image, ignore_missing=True):
         """Delete an image
@@ -83,13 +277,13 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.compute.v2.image.ImageDetail` objects,
             otherwise :class:`~openstack.compute.v2.image.Image`.
             *Default: ``True``*
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
+        :param kwargs query: Optional query parameters to be sent to limit
                                  the resources being returned.
 
         :returns: A generator of image objects
         """
         img = _image.ImageDetail if details else _image.Image
-        return self._list(img, paginated=True, **query)
+        return self._list(img, **query)
 
     def _get_base_resource(self, res, base):
         # Metadata calls for Image and Server can work for both those
@@ -215,7 +409,7 @@ class Proxy(proxy.Proxy):
         :returns: A generator of keypair objects
         :rtype: :class:`~openstack.compute.v2.keypair.Keypair`
         """
-        return self._list(_keypair.Keypair, paginated=False)
+        return self._list(_keypair.Keypair)
 
     def get_limits(self):
         """Retrieve limits that are applied to the project's account
@@ -227,6 +421,18 @@ class Proxy(proxy.Proxy):
         """
         return self._get(limits.Limits)
 
+    def create_server(self, **attrs):
+        """Create a new server from attributes
+
+        :param dict attrs: Keyword arguments which will be used to create
+                           a :class:`~openstack.compute.v2.server.Server`,
+                           comprised of the properties on the Server class.
+
+        :returns: The results of server creation
+        :rtype: :class:`~openstack.compute.v2.server.Server`
+        """
+        return self._create(_server.Server, **attrs)
+
     def delete_server(self, server, ignore_missing=True, force=False):
         """Delete a server
 
@@ -246,10 +452,35 @@ class Proxy(proxy.Proxy):
             server = self._get_resource(_server.Server, server)
             server.force_delete(self)
         else:
-            self._generated_delete_server(
-                server, ignore_missing=ignore_missing)
+            self._delete(_server.Server, server, ignore_missing=ignore_missing)
 
-    def servers(self, details=True, **query):
+    def find_server(self, name_or_id, ignore_missing=True):
+        """Find a single server
+
+        :param name_or_id: The name or ID of a server.
+        :param bool ignore_missing: When set to ``False``
+                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    raised when the resource does not exist.
+                    When set to ``True``, None will be returned when
+                    attempting to find a nonexistent resource.
+        :returns: One :class:`~openstack.compute.v2.server.Server` or None
+        """
+        return self._find(_server.Server, name_or_id,
+                          ignore_missing=ignore_missing)
+
+    def get_server(self, server):
+        """Get a single server
+
+        :param server: The value can be the ID of a server or a
+                       :class:`~openstack.compute.v2.server.Server` instance.
+
+        :returns: One :class:`~openstack.compute.v2.server.Server`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        return self._get(_server.Server, server)
+
+    def servers(self, details=True, all_projects=False, **query):
         """Retrieve a generator of servers
 
         :param bool details: When set to ``False``
@@ -257,7 +488,7 @@ class Proxy(proxy.Proxy):
                     will be returned. The default, ``True``, will cause
                     :class:`~openstack.compute.v2.server.ServerDetail`
                     instances to be returned.
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
+        :param kwargs query: Optional query parameters to be sent to limit
             the servers being returned.  Available parameters include:
 
             * changes_since: A time/date stamp for when the server last changed
@@ -273,6 +504,8 @@ class Proxy(proxy.Proxy):
             * status: Value of the status of the server so that you can filter
                     on "ACTIVE" for example.
             * host: Name of the host as a string.
+            * all_projects: Flag to request servers be returned from all
+                            projects, not just the currently scoped one.
             * limit: Requests a specified page size of returned items from the
                      query.  Returns a number of items up to the specified
                      limit value. Use the limit parameter to make an initial
@@ -286,7 +519,10 @@ class Proxy(proxy.Proxy):
 
         :returns: A generator of server instances.
         """
-        return self._generated_servers(details=details, **query)
+        if all_projects:
+            query['all_projects'] = True
+        srv = _server.ServerDetail if details else _server.Server
+        return self._list(srv, **query)
 
     def update_server(self, server, **attrs):
         """Update a server
@@ -692,8 +928,29 @@ class Proxy(proxy.Proxy):
         server = self._get_resource(_server.Server, server)
         return server.get_console_output(self, length=length)
 
-    def wait_for_server(self, server, status='ACTIVE', failures=['ERROR'],
+    def wait_for_server(self, server, status='ACTIVE', failures=None,
                         interval=2, wait=120):
+        """Wait for a server to be in a particular status.
+
+        :param res: The resource to wait on to reach the specified status.
+                    The resource must have a ``status`` attribute.
+        :type resource: A :class:`~openstack.resource.Resource` object.
+        :param status: Desired status.
+        :param failures: Statuses that would be interpreted as failures.
+        :type failures: :py:class:`list`
+        :param interval: Number of seconds to wait before to consecutive
+                         checks. Default to 2.
+        :param wait: Maximum number of seconds to wait before the change.
+                     Default to 120.
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+                 to the desired status failed to occur in specified seconds.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
+                 has transited to one of the failure statuses.
+        :raises: :class:`~AttributeError` if the resource does not have a
+                ``status`` attribute.
+        """
+        failures = ['ERROR'] if failures is None else failures
         return resource.wait_for_status(
             self, server, status, failures, interval, wait)
 
@@ -777,7 +1034,7 @@ class Proxy(proxy.Proxy):
         :rtype: :class:`~openstack.compute.v2.server_interface.ServerInterface`
         """
         server_id = resource.Resource._get_id(server)
-        return self._list(_server_interface.ServerInterface, paginated=False,
+        return self._list(_server_interface.ServerInterface,
                           server_id=server_id)
 
     def server_ips(self, server, network_label=None):
@@ -792,7 +1049,7 @@ class Proxy(proxy.Proxy):
         :rtype: :class:`~openstack.compute.v2.server_ip.ServerIP`
         """
         server_id = resource.Resource._get_id(server)
-        return self._list(server_ip.ServerIP, paginated=False,
+        return self._list(server_ip.ServerIP,
                           server_id=server_id, network_label=network_label)
 
     def availability_zones(self, details=False):
@@ -811,7 +1068,7 @@ class Proxy(proxy.Proxy):
         else:
             az = availability_zone.AvailabilityZone
 
-        return self._list(az, paginated=False)
+        return self._list(az)
 
     def get_server_metadata(self, server):
         """Return a dictionary of metadata for a server
@@ -929,22 +1186,31 @@ class Proxy(proxy.Proxy):
     def server_groups(self, **query):
         """Return a generator of server groups
 
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
+        :param kwargs query: Optional query parameters to be sent to limit
                                  the resources being returned.
 
         :returns: A generator of ServerGroup objects
         :rtype: :class:`~openstack.compute.v2.server_group.ServerGroup`
         """
-        return self._list(_server_group.ServerGroup, paginated=False, **query)
+        return self._list(_server_group.ServerGroup, **query)
 
-    def hypervisors(self):
+    def hypervisors(self, details=False):
         """Return a generator of hypervisor
 
+        :param bool details: When set to the default, ``False``,
+                    :class:`~openstack.compute.v2.hypervisor.Hypervisor`
+                    instances will be returned. ``True`` will cause
+                    :class:`~openstack.compute.v2.hypervisor.HypervisorDetail`
+                    instances to be returned.
         :returns: A generator of hypervisor
         :rtype: class: `~openstack.compute.v2.hypervisor.Hypervisor`
         """
+        if details:
+            hypervisor = _hypervisor.HypervisorDetail
+        else:
+            hypervisor = _hypervisor.Hypervisor
 
-        return self._list(_hypervisor.Hypervisor, paginated=False)
+        return self._list(hypervisor)
 
     def find_hypervisor(self, name_or_id, ignore_missing=True):
         """Find a hypervisor from name or id to get the corresponding info
@@ -1023,7 +1289,7 @@ class Proxy(proxy.Proxy):
         :rtype: class: `~openstack.compute.v2.service.Service`
         """
 
-        return self._list(_service.Service, paginated=False)
+        return self._list(_service.Service)
 
     def create_volume_attachment(self, server, **attrs):
         """Create a new volume attachment from attributes
@@ -1145,7 +1411,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.compute.v2.volume_attachment.VolumeAttachment`
         """
         server_id = resource.Resource._get_id(server)
-        return self._list(_volume_attachment.VolumeAttachment, paginated=False,
+        return self._list(_volume_attachment.VolumeAttachment,
                           server_id=server_id)
 
     def migrate_server(self, server):
@@ -1158,15 +1424,63 @@ class Proxy(proxy.Proxy):
         server = self._get_resource(_server.Server, server)
         server.migrate(self)
 
-    def live_migrate_server(self, server, host=None, force=False):
-        """Migrate a server from one host to target host
+    def live_migrate_server(
+            self, server, host=None, force=False, block_migration=None):
+        """Live migrate a server from one host to target host
 
-        :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` instance.
-        :param host: The host to which to migrate the server
-        :param force: Force a live-migration by not verifying the provided
-                      destination host by the scheduler.
+        :param server:
+            Either the ID of a server or a
+            :class:`~openstack.compute.v2.server.Server` instance.
+        :param str host:
+            The host to which to migrate the server. If the Nova service is
+            too old, the host parameter implies force=True which causes the
+            Nova scheduler to be bypassed. On such clouds, a ``ValueError``
+            will be thrown if ``host`` is given without ``force``.
+        :param bool force:
+            Force a live-migration by not verifying the provided destination
+            host by the scheduler. This is unsafe and not recommended.
+        :param block_migration:
+            Perform a block live migration to the destination host by the
+            scheduler.  Can be 'auto', True or False. Some clouds are too old
+            to support 'auto', in which case a ValueError will be thrown. If
+            omitted, the value will be 'auto' on clouds that support it, and
+            False on clouds that do not.
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.live_migrate(self, host, force)
+        server.live_migrate(
+            self, host,
+            force=force,
+            block_migration=block_migration)
+
+    def wait_for_delete(self, res, interval=2, wait=120):
+        """Wait for a resource to be deleted.
+
+        :param res: The resource to wait on to be deleted.
+        :type resource: A :class:`~openstack.resource.Resource` object.
+        :param interval: Number of seconds to wait before to consecutive
+                         checks. Default to 2.
+        :param wait: Maximum number of seconds to wait before the change.
+                     Default to 120.
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+                 to delete failed to occur in the specified seconds.
+        """
+        return resource.wait_for_delete(self, res, interval, wait)
+
+    def get_server_diagnostics(self, server):
+        """Get a single server diagnostics
+
+        :param server: This parameter need to be specified when ServerInterface
+                       ID is given as value. It can be either the ID of a
+                       server or a :class:`~openstack.compute.v2.server.Server`
+                       instance that the interface belongs to.
+
+        :returns: One
+            :class:`~openstack.compute.v2.server_diagnostics.ServerDiagnostics`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        server_id = self._get_resource(_server.Server, server).id
+        return self._get(_server_diagnostics.ServerDiagnostics,
+                         server_id=server_id, requires_id=False)
