@@ -10,26 +10,23 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack.compute import compute_service
-from openstack import resource2
+from openstack import resource
 
 
-class AvailabilityZone(resource2.Resource):
+class AvailabilityZone(resource.Resource):
     resources_key = 'availabilityZoneInfo'
     base_path = '/os-availability-zone'
 
-    service = compute_service.ComputeService()
-
     # capabilities
     allow_list = True
 
     # Properties
     #: name of availability zone
-    name = resource2.Body('zoneName')
+    name = resource.Body('zoneName')
     #: state of availability zone
-    state = resource2.Body('zoneState')
+    state = resource.Body('zoneState')
     #: hosts of availability zone
-    hosts = resource2.Body('hosts')
+    hosts = resource.Body('hosts')
 
 
 class AvailabilityZoneDetail(AvailabilityZone):
