@@ -10,7 +10,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack.identity import identity_service
 from openstack import resource
 
 
@@ -18,15 +17,14 @@ class Endpoint(resource.Resource):
     resource_key = 'endpoint'
     resources_key = 'endpoints'
     base_path = '/endpoints'
-    service = identity_service.IdentityService()
 
     # capabilities
     allow_create = True
-    allow_get = True
-    allow_update = True
+    allow_fetch = True
+    allow_commit = True
     allow_delete = True
     allow_list = True
-    update_method = 'PATCH'
+    commit_method = 'PATCH'
 
     _query_mapping = resource.QueryParameters(
         'interface', 'service_id',
