@@ -10,15 +10,13 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack.identity import identity_service
-from openstack import resource2 as resource
+from openstack import resource
 
 
 class RoleDomainGroupAssignment(resource.Resource):
     resource_key = 'role'
     resources_key = 'roles'
     base_path = '/domains/%(domain_id)s/groups/%(group_id)s/roles'
-    service = identity_service.IdentityService()
 
     # capabilities
     allow_list = True
