@@ -10,8 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack.network import network_service
-from openstack import resource2 as resource
+from openstack import resource
 
 
 class AddressScope(resource.Resource):
@@ -19,12 +18,11 @@ class AddressScope(resource.Resource):
     resource_key = 'address_scope'
     resources_key = 'address_scopes'
     base_path = '/address-scopes'
-    service = network_service.NetworkService()
 
     # capabilities
     allow_create = True
-    allow_get = True
-    allow_update = True
+    allow_fetch = True
+    allow_commit = True
     allow_delete = True
     allow_list = True
 
