@@ -10,7 +10,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import json
 import operator
 
 from keystoneauth1 import adapter
@@ -37,6 +36,9 @@ EXAMPLE = {
     'status': '8',
     'tags': ['g', 'h', 'i'],
     'updated_at': '2015-03-09T12:15:57.233772',
+    'os_hash_algo': 'sha512',
+    'os_hash_value': '073b4523583784fbe01daff81eba092a262ec3',
+    'os_hidden': False,
     'virtual_size': '10',
     'visibility': '11',
     'location': '12',
@@ -45,8 +47,6 @@ EXAMPLE = {
     'file': '15',
     'locations': ['15', '16'],
     'direct_url': '17',
-    'path': '18',
-    'value': '19',
     'url': '20',
     'metadata': {'21': '22'},
     'architecture': '23',
@@ -79,16 +79,23 @@ EXAMPLE = {
     'vmware_ostype': '48',
     'auto_disk_config': True,
     'os_type': '49',
+    'os_admin_user': 'ubuntu',
+    'hw_qemu_guest_agent': True,
+    'os_require_quiesce': True,
 }
 
 
 class FakeResponse(object):
-    def __init__(self, response, status_code=200, headers=None):
+    def __init__(self, response, status_code=200, headers=None, reason=None):
         self.body = response
         self.content = response
         self.status_code = status_code
         headers = headers if headers else {'content-type': 'application/json'}
         self.headers = requests.structures.CaseInsensitiveDict(headers)
+        if reason:
+            self.reason = reason
+        # for the sake of "list" response faking
+        self.links = []
 
     def json(self):
         return self.body
@@ -103,19 +110,43 @@ class TestImage(base.TestCase):
         self.resp.json = mock.Mock(return_value=self.resp.body)
         self.sess = mock.Mock(spec=adapter.Adapter)
         self.sess.post = mock.Mock(return_value=self.resp)
+        self.sess.put = mock.Mock(return_value=FakeResponse({}))
+        self.sess.delete = mock.Mock(return_value=FakeResponse({}))
+        self.sess.fetch = mock.Mock(return_value=FakeResponse({}))
+        self.sess.default_microversion = None
+        self.sess.retriable_status_codes = None
 
     def test_basic(self):
         sot = image.Image()
         self.assertIsNone(sot.resource_key)
         self.assertEqual('images', sot.resources_key)
         self.assertEqual('/images', sot.base_path)
-        self.assertEqual('image', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
+        self.assertDictEqual({'created_at': 'created_at',
+                              'is_hidden': 'os_hidden',
+                              'limit': 'limit',
+                              'marker': 'marker',
+                              'member_status': 'member_status',
+                              'name': 'name',
+                              'owner': 'owner',
+                              'protected': 'protected',
+                              'size_max': 'size_max',
+                              'size_min': 'size_min',
+                              'sort': 'sort',
+                              'sort_dir': 'sort_dir',
+                              'sort_key': 'sort_key',
+                              'status': 'status',
+                              'tag': 'tag',
+                              'updated_at': 'updated_at',
+                              'visibility': 'visibility'
+                              },
+                             sot._query_mapping._mapping)
+
     def test_make_it(self):
         sot = image.Image(**EXAMPLE)
         self.assertEqual(IDENTIFIER, sot.id)
@@ -131,6 +162,9 @@ class TestImage(base.TestCase):
         self.assertEqual(EXAMPLE['status'], sot.status)
         self.assertEqual(EXAMPLE['tags'], sot.tags)
         self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)
+        self.assertEqual(EXAMPLE['os_hash_algo'], sot.hash_algo)
+        self.assertEqual(EXAMPLE['os_hash_value'], sot.hash_value)
+        self.assertEqual(EXAMPLE['os_hidden'], sot.is_hidden)
         self.assertEqual(EXAMPLE['virtual_size'], sot.virtual_size)
         self.assertEqual(EXAMPLE['visibility'], sot.visibility)
         self.assertEqual(EXAMPLE['size'], sot.size)
@@ -138,8 +172,6 @@ class TestImage(base.TestCase):
         self.assertEqual(EXAMPLE['file'], sot.file)
         self.assertEqual(EXAMPLE['locations'], sot.locations)
         self.assertEqual(EXAMPLE['direct_url'], sot.direct_url)
-        self.assertEqual(EXAMPLE['path'], sot.path)
-        self.assertEqual(EXAMPLE['value'], sot.value)
         self.assertEqual(EXAMPLE['url'], sot.url)
         self.assertEqual(EXAMPLE['metadata'], sot.metadata)
         self.assertEqual(EXAMPLE['architecture'], sot.architecture)
@@ -175,6 +207,10 @@ class TestImage(base.TestCase):
         self.assertEqual(EXAMPLE['vmware_ostype'], sot.vmware_ostype)
         self.assertEqual(EXAMPLE['auto_disk_config'], sot.has_auto_disk_config)
         self.assertEqual(EXAMPLE['os_type'], sot.os_type)
+        self.assertEqual(EXAMPLE['os_admin_user'], sot.os_admin_user)
+        self.assertEqual(EXAMPLE['hw_qemu_guest_agent'],
+                         sot.hw_qemu_guest_agent)
+        self.assertEqual(EXAMPLE['os_require_quiesce'], sot.os_require_quiesce)
 
     def test_deactivate(self):
         sot = image.Image(**EXAMPLE)
@@ -194,7 +230,7 @@ class TestImage(base.TestCase):
         sot = image.Image(**EXAMPLE)
         tag = "lol"
 
-        self.assertIsNone(sot.add_tag(self.sess, tag))
+        sot.add_tag(self.sess, tag)
         self.sess.put.assert_called_with(
             'images/IDENTIFIER/tags/%s' % tag,
         )
@@ -203,11 +239,28 @@ class TestImage(base.TestCase):
         sot = image.Image(**EXAMPLE)
         tag = "lol"
 
-        self.assertIsNone(sot.remove_tag(self.sess, tag))
+        sot.remove_tag(self.sess, tag)
         self.sess.delete.assert_called_with(
             'images/IDENTIFIER/tags/%s' % tag,
         )
 
+    def test_import_image(self):
+        sot = image.Image(**EXAMPLE)
+        json = {"method": {"name": "web-download", "uri": "such-a-good-uri"}}
+        sot.import_image(self.sess, "web-download", "such-a-good-uri")
+        self.sess.post.assert_called_with(
+            'images/IDENTIFIER/import',
+            json=json
+        )
+
+    def test_import_image_with_uri_not_web_download(self):
+        sot = image.Image(**EXAMPLE)
+        self.assertRaises(exceptions.InvalidRequest,
+                          sot.import_image,
+                          self.sess,
+                          "glance-direct",
+                          "such-a-good-uri")
+
     def test_upload(self):
         sot = image.Image(**EXAMPLE)
 
@@ -259,7 +312,7 @@ class TestImage(base.TestCase):
         self.sess.get.assert_has_calls(
             [mock.call('images/IDENTIFIER/file',
                        stream=False),
-             mock.call('images/IDENTIFIER',)])
+             mock.call('images/IDENTIFIER', microversion=None)])
 
         self.assertEqual(rv, resp1.content)
 
@@ -285,7 +338,7 @@ class TestImage(base.TestCase):
         self.sess.get.assert_has_calls(
             [mock.call('images/IDENTIFIER/file',
                        stream=False),
-             mock.call('images/IDENTIFIER',)])
+             mock.call('images/IDENTIFIER', microversion=None)])
 
         self.assertEqual(rv, resp1.content)
 
@@ -304,7 +357,9 @@ class TestImage(base.TestCase):
         self.assertEqual(rv, resp)
 
     def test_image_update(self):
-        sot = image.Image(**EXAMPLE)
+        values = EXAMPLE.copy()
+        del values['instance_uuid']
+        sot = image.Image.existing(**values)
         # Let the translate pass through, that portion is tested elsewhere
         sot._translate_response = mock.Mock()
 
@@ -318,21 +373,45 @@ class TestImage(base.TestCase):
         resp.status_code = 200
         self.sess.patch.return_value = resp
 
-        value = ('[{"value": "fake_name", "op": "replace", "path": "/name"}, '
-                 '{"value": "fake_value", "op": "add", '
-                 '"path": "/new_property"}]')
-        fake_img = sot.to_dict()
-        fake_img['name'] = 'fake_name'
-        fake_img['new_property'] = 'fake_value'
+        value = [{"value": "fake_name", "op": "replace", "path": "/name"},
+                 {"value": "fake_value", "op": "add",
+                  "path": "/instance_uuid"}]
 
-        sot.update(self.sess, **fake_img)
+        sot.name = 'fake_name'
+        sot.instance_uuid = 'fake_value'
+        sot.commit(self.sess)
         url = 'images/' + IDENTIFIER
         self.sess.patch.assert_called_once()
         call = self.sess.patch.call_args
         call_args, call_kwargs = call
         self.assertEqual(url, call_args[0])
         self.assertEqual(
-            sorted(json.loads(value), key=operator.itemgetter('value')),
-            sorted(
-                json.loads(call_kwargs['data']),
-                key=operator.itemgetter('value')))
+            sorted(value, key=operator.itemgetter('value')),
+            sorted(call_kwargs['json'], key=operator.itemgetter('value')))
+
+    def test_image_find(self):
+        sot = image.Image()
+
+        self.sess._get_connection = mock.Mock(return_value=self.cloud)
+        self.sess.get.side_effect = [
+            # First fetch by name
+            FakeResponse(None, 404, headers={}, reason='dummy'),
+            # Then list with no results
+            FakeResponse({'images': []}),
+            # And finally new list of hidden images with one searched
+            FakeResponse({'images': [EXAMPLE]})
+
+        ]
+
+        result = sot.find(self.sess, EXAMPLE['name'])
+
+        self.sess.get.assert_has_calls([
+            mock.call('images/' + EXAMPLE['name'], microversion=None),
+            mock.call('/images', headers={'Accept': 'application/json'},
+                      microversion=None, params={}),
+            mock.call('/images', headers={'Accept': 'application/json'},
+                      microversion=None, params={'os_hidden': True})
+        ])
+
+        self.assertIsInstance(result, image.Image)
+        self.assertEqual(IDENTIFIER, result.id)
